/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.5)

   class      : JavaNotesServer
   date       : 11.10.97
   description: the top level class of the JavaNotes Server

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.io.*;
import java.net.*;
import java.util.*;

/* ******************************************************************
   top level class declaration
*********************************************************************/

public class JavaNotesServer {

  // class variable declarations
  final static String usage      = "usage: java JavaNotesServer -p:Port# "+
                                   " [-m:MaxConnections#] [-f:Passwordfile]"+
                                   "\n       [-d:Datadirectory]";
  final static String dupPar     = "Duplicate parameter";
  final static String invNumeric = "Invalid number format in ";
  final static String invFile    = " is not an existing readable file";
  final static String invDir     = " is not an existing readable directory";

  // methods
  public static void main(String[] args) {
    DocVer docVer      = new DocVer("JavaNotes Server","1.0");
    boolean err        = false;
    int port           = -1;
    int maxCon         = -1;
    String pwdFilename = null;
    String dataDir     = null;
    int i;
    File pwdFile;
    File dirFile;
    PwdTable pwdTable;
    FilePool filePool;

    System.out.println(docVer.getHeadline());
    if (args.length < 1) { System.out.println(usage); return; }
    for (i=0; i<args.length; i++) {
      if (args[i].startsWith("-p:")) {
        if (port >= 0) fail(null,dupPar);
        try {
          port = Integer.parseInt(args[i].substring(3));
        } catch (NumberFormatException excep) {
          err = true;
        }
        if (err || (port < 0)) fail(null,invNumeric+'"'+args[i]+'"');
      }
      else if (args[i].startsWith("-m:")) {
        if (maxCon >= 0) fail(null,dupPar);
        maxCon = Integer.parseInt(args[i].substring(3));
        try {
          maxCon = Integer.parseInt(args[i].substring(3));
        } catch (NumberFormatException excep) {
          err = true;
        }
        if (err || (maxCon < 1)) fail(null,invNumeric+'"'+args[i]+'"');
      }
      else if (args[i].startsWith("-f:")) {
        if (pwdFilename != null) fail(null,dupPar);
        pwdFilename = args[i].substring(3);
      }
      else if (args[i].startsWith("-d:")) {
        if (dataDir != null) fail(null,dupPar);
        dataDir = args[i].substring(3);
      }
      else {
        System.out.println(usage); return;
      } // end if
    } // end for
    if (port < 0) { System.out.println(usage); return; }
    if (maxCon < 1) maxCon = 5;        // assign default values
    if (pwdFilename == null) pwdFilename = "passwd";
    if (dataDir == null) dataDir = ".";
    pwdFile = new File(pwdFilename);
    if (!(pwdFile.exists() && pwdFile.isFile() && pwdFile.canRead()))
      fail(null,'"'+pwdFilename+'"'+invFile);
    // end if
    dirFile = new File(dataDir);
    if (!(dirFile.exists() && dirFile.isDirectory() && dirFile.canRead()))
      fail(null,'"'+dataDir+'"'+invDir);
    // end if
    try {
      pwdTable = new PwdTable(pwdFile);
    } catch (IOException excep) {
      fail(excep,"Could not load user information");
      pwdTable = null;
    }
    System.out.println("User information loaded from file "+'"'+pwdFile+'"');
    filePool = new FilePool(docVer,dirFile);
    System.out.println("File pool initialized in directory "+'"'+dirFile+'"');
    //try { System.out.println(InetAddress.getLocalHost());
    //} catch (UnknownHostException excep) {
    //  fail(excep,"Shit!");
    //}
    ListeningThread listeningThread = new ListeningThread(port,maxCon,
                                                          pwdTable,filePool);
    OperatorThread  operatorThread  = new OperatorThread(filePool);
  } // end main

  // methods
  static void fail(Exception excep, String msg) {
    if (excep == null)
      System.err.println(msg);
    else
      System.err.println(msg+"."+excep);
    // end if
    System.exit(1);
  } // end fail

} // end JavaNotesServer

/* ******************************************************************
   classes
*********************************************************************/

class ListeningThread extends Thread {

  // instance variable declarations
  protected ServerSocket serverListeningSocket;
  private PwdTable pwdTable;
  private FilePool filePool;

  // constructors
  ListeningThread(int port, int maxCon,
                  PwdTable pwdTable, FilePool filePool) {

    this.pwdTable = pwdTable;
    this.filePool = filePool;
    try {
      serverListeningSocket = new ServerSocket(port,maxCon);
    } catch (IOException excep) {
      fail(excep,"Could not start Listening Thread");
    }
    System.out.println("Listening Thread using port "+port+
                       " with a maximum of "+maxCon+" connections started");
    this.setDaemon(true);
    this.start();
  } // end constructor ListeningThread

  // methods
  static void fail(Exception excep, String msg) {
    System.err.println(msg+"."+excep);
    System.exit(1);
  } // end fail

  public void run() {
    try {
      while (true) {
        Socket clientSocket = serverListeningSocket.accept();
        ConnectionThread connection = new ConnectionThread(clientSocket,
                                            pwdTable,filePool);
      } // end while
    } catch (IOException excep) {
      fail(excep,"Listening loop failed");
    }
  } // end run

} // end ListeningTread

//*******************************************************************

class OperatorThread extends Thread {

  // instance variable declarations
  private FilePool filePool;

  // constructors
  OperatorThread(FilePool filePool) {

    this.filePool = filePool;
    System.out.println("Operator Thread started (Enter "+'"'+'?'+'"'+
                       " to get a list of available commands)");
    this.start();
  } // end constructor OperatorThread

  // methods
  public void run() {
    String line = "";
    String cmd;
    StringTokenizer strTok;
    BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));

    do {
      System.out.print(">"); System.out.flush();
      try {
        line = reader.readLine();
      } catch (IOException excep) {
        excep.printStackTrace();
      }
      strTok = new StringTokenizer(line);
      if (strTok.hasMoreTokens()) {
        cmd = strTok.nextToken().toLowerCase();
        if (strTok.hasMoreTokens()) {
          cmd = "";
          System.out.println("Illegal parameter(s)");
        } // end if
      }
      else {
        cmd = "";
      } // end if
      if (cmd.equals("")) {
        // on empty command do nothing
      } else if (cmd.equals("?") || cmd.equals("help")) {
        printHelp();
      } else if (cmd.equals("exit")) {
        System.out.println("C'ya!");
      } else if (cmd.equals("files")) {
        System.out.println(filePool.getNrOfFiles()+
                           " file(s)");
      } else if (cmd.equals("connections")) {
        System.out.println(ConnectionThread.getNumberOfConnections()+
                           " connection(s)");
      } else {
        System.out.println("Illegal command");
      } // end if
    } while(!line.equals("exit"));
  } // end run

  public void printHelp() {
    System.out.println("Available commands:");
    System.out.println("  help        - this list of available commands");
    System.out.println("  exit        - shutdown JavaNotes Server");
    System.out.println("  files       - number of open files");
    System.out.println("  connections - number of open connections");
  } // end printHelp

} // end OperatorThread

/* ******************************************************************
   end of file
*********************************************************************/
