/* ******************************************************************

   author     : puehringer edgar

   compiler   : javac (jdk 1.1.5)

   class      : WeeklyPanel
   date       : 03.01.98
   description: an panel that displays the week view of the calendar
   modified   : 19.02.98 - itemStateChanged calculates start and
                           end time of selection

*********************************************************************/

/* ******************************************************************
   import statements
*********************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.SimpleDateFormat;

/* ******************************************************************
   top level class declaration
*********************************************************************/

public class WeeklyPanel extends Panel
  implements Observer,ItemListener {

  // class variable declarations
  private final static int MINUTES_PER_CELL   = 30;
  private final static int TIME_WIDTH         = 40;
  private final static int ENTRY_WIDTH        = 70;
  private final static int[] A_COL_WIDTH      = {TIME_WIDTH,ENTRY_WIDTH,
                                                 ENTRY_WIDTH,ENTRY_WIDTH,
                                                 ENTRY_WIDTH,ENTRY_WIDTH,
                                                 ENTRY_WIDTH,ENTRY_WIDTH
                                                };

  // instance variable declarations
  private LoginInfoModel lModel;
  private ScopeModel sModel;
  private DateListModel dModel;
  private int vSize;
  private SpreadSheetCanvas spreadSheet;
  private SimpleDateFormat formatter;
  private Calendar calendar;

  // constructors
  public WeeklyPanel(LoginInfoModel lModel, ScopeModel sModel,
                    DateListModel dModel, int vSize) {
    formatter = new SimpleDateFormat("H:mm",CalendarCreator.getLocale());
    int i,n;

    this.lModel = lModel;
    this.sModel = sModel;
    this.dModel = dModel;
    this.sModel.addObserver(this);
    this.dModel.addObserver(this);
    this.vSize = vSize;

    calendar = CalendarCreator.get();

    ScrollPane sp = new ScrollPane(ScrollPane.SCROLLBARS_AS_NEEDED);
    sp.setSize(TIME_WIDTH+7*ENTRY_WIDTH+sp.getVScrollbarWidth(),vSize);
    spreadSheet = new SpreadSheetCanvas(24*60/MINUTES_PER_CELL+1,
                                        A_COL_WIDTH,
                                        SystemColor.control,true);
    sp.add(spreadSheet);
    calendar.setTime(sModel.getDate());
    spreadSheet.setCell(0,0,0,"<");
    spreadSheet.setCell(0,0,1,">");
    n = 24*60/MINUTES_PER_CELL;
    for (i=0; i<n; i++) {
      spreadSheet.setCell(0,i+1,0,formatter.format(calendar.getTime()));
      calendar.add(Calendar.MINUTE,MINUTES_PER_CELL);
    } // end for
    formatter = new SimpleDateFormat("EE, dd.MM.yy",CalendarCreator.getLocale());
    calendar.setTime(sModel.getStartOfScope());
    for (i=1; i<8; i++) {
      spreadSheet.setCell(i,0,0,formatter.format(calendar.getTime()));
      calendar.add(Calendar.DATE,1);
    } // end for
    add(sp);
    spreadSheet.addItemListener(this);
  } // end constructor WeeklyPanel

  // methods
  public void update(Observable obs, Object arg) {
    if (sModel.getScopeWidth() != ScopeModel.WEEKLY) {
      spreadSheet.setBufferEnabled(false);
      return;
    }
    else {
      spreadSheet.setBufferEnabled(true);
    } // end if

    if (obs.equals(dModel)) {
      if (dModel.isNew()) {
        // update spreadsheet
        Enumeration enum = dModel.getCalendarEntries();
        CalendarEntry entry;
        Date oldDate;
        int n = 24*60/MINUTES_PER_CELL;
        int i,l;

        // update column header
        calendar.setTime(sModel.getStartOfScope());
        for (i=1; i<8; i++) {
          spreadSheet.setCell(i,0,0,formatter.format(calendar.getTime()));
          calendar.add(Calendar.DATE,1);
        } // end for

        // delete spreadsheet
        spreadSheet.unSelect();
        for (i=0; i<n*7; i++) {
          spreadSheet.deleteCell((i/n)+1,(i%n)+1);
        } // end for

        // set calendar to start
        calendar.setTime(sModel.getStartOfScope());
        oldDate = calendar.getTime();
        calendar.add(Calendar.MINUTE,MINUTES_PER_CELL);
        i = 0; l = 0;
        while (enum.hasMoreElements()) {
          entry = (CalendarEntry)enum.nextElement();
          if (entry.getStart().before(oldDate)) {
            Assert.out.println("WeeklyPanel: illegal date in list");
            continue;
          } // end if
          while (!calendar.getTime().after(entry.getStart())) {
            oldDate = calendar.getTime();
            calendar.add(Calendar.MINUTE,MINUTES_PER_CELL);
            i++; l = 0;
          } // end while
          if (i >= n*7) {
            Assert.out.println("WeeklyPanel: date after end of spreadsheet");
            break;
          } // end if
          spreadSheet.setCell((i/n)+1,(i%n)+1,l,entry);
          l++;
          while ((entry.getEnd().after(calendar.getTime())) && (i < (n*7-1))) {
            oldDate = calendar.getTime();
            calendar.add(Calendar.MINUTE,MINUTES_PER_CELL);
            i++; l = 1;
            if ((i % n) == 0)
              spreadSheet.setCell((i/n)+1,(i%n)+1,0,entry);
            else
              spreadSheet.setUplinkCell((i/n)+1,(i%n)+1);
            // end if
          } // end while
        } // end while
        spreadSheet.repaint();
      } // end if
    } // end if
  } // end update

  public void itemStateChanged(ItemEvent e) {
    Object aObj[] = spreadSheet.getSelectedObjects();
    Point point   = spreadSheet.getSelectedIndex();

    if (point == null) {
      sModel.setSelectedObject(aObj[0]);
    }
    else if ((point.x == 0) && (point.y == 0)) {
      int diff = (spreadSheet.getSelectedLayer() == 0) ? -7 : 7;

      calendar.setTime(sModel.getDate());
      calendar.add(Calendar.DATE,diff);
      sModel.setDate(calendar.getTime());
    }
    else if ((point.x == 0) || (point.y == 0)) {
      spreadSheet.unSelect();
      sModel.setSelectedObject(aObj[0]);
    }
    else {
      calendar.setTime(sModel.getStartOfScope());
      calendar.add(Calendar.DATE,point.x-1);
      calendar.add(Calendar.MINUTE,MINUTES_PER_CELL*(point.y-1));
      Date start = new Date(calendar.getTime().getTime());
      calendar.add(Calendar.MINUTE,MINUTES_PER_CELL);
      Date end = new Date(calendar.getTime().getTime());
      sModel.setSelectedObject(aObj[0],start,end);
    } // end if
  } // end itemStateChanged

} // end WeeklyPanel

/* ******************************************************************
   end of file
*********************************************************************/
